#!/usr/bin/perl
# nagios: -epn
## $Id: check_nut by J.Ph. LEMOINE$

## Nagios Plugin for check UPS via nut
##
## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.
##
## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
##
## You should have received a copy of the GNU General Public License
## along with this program; if not, write to the Free Software
## Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

# NB : on ne peut pas utiliser le fichier ini : tout est en anglais
# NB : we cannot use the ini file : all is in english

use strict;
use warnings;
use Switch;

sub try_load_module($) {
    my $module = shift;

    eval "require $module";
    if ($@) {
     print "DOMAIN UNKNOWN - Can't load module $module, ",
     "perhaps it isn't installed\n$@";
     print "Config::INI::Reader --> sudo apt-get install libconfig-ini-perl\n";
     print "Nagios::Plugin      --> sudo apt-get install libnagios-pluging-perl\n";
     exit 3;
    }

    $module->import;
}

BEGIN {
    try_load_module 'Config::INI::Reader';
    try_load_module 'Nagios::Plugin';
}

use vars qw/ $VERSION /;
use lib "/usr/lib/perl5/B";

my $VERSION = '1.0';
my $LICENCE = "Ce plugin Nagios est gratuit et libre de droits, et vous pouvez l'utiliser à votre convenance."
  . ' Il est livré avec ABSOLUMENT AUCUNE GARANTIE.';

my $plugin_nagios = Nagios::Plugin->new(
  shortname => 'check_nut',
  usage     => 'Usage : %s -t|--type : <load|batterie[temps|statut> -w|--warning <warning value> -c|--critical <critical value> -l|--lang <language code fr or en> -i|--ini_path <path ini file>',
  version   => $VERSION,
  license   => $LICENCE,
);

# put the 2 letters of the country code
# in 10/28/2018, there is 2 possibilies :
# fr for french
# en for english
# can be changed if needed
my $country_code_def="fr";

# start with a / is absolute path
# with 1st directory if relative path
# can be changed if needed
my $path_to_ini="ini";

# variables declaration
# do not modify

# script pamameter
my $TYPE         = '';
my $WARN         = '';
my $CRIT         = '';
my $COUNTRY_CODE = '';
my $INI_PATH     = '';
# name of the perl script
my $PROGNAME     = '';
# path to perl script
my $PATH         = '';
# name of the ini file
my $NAME_INI     = '';
# content of the ini file = traduction
my $hash_ref     = '';
# path & name oh the ini file
my $ini_file     = '';

# define the current path as possible search path
my $PATH_PL = '';

my $pgm            = '';
my $pgm_tot        = '';
my $text01a        = '';
my $text01b        = '';
my $ups_name       = '';
my $server_IP      = '';
my $ups_total      = '';
my $arg            = '';
my $res            = '';
my $res_tra        = '';
my $temp_tra       = '';
my $nb_hours       = '';
my $nb_mins        = '';
my $result_msg     = '';
my $notif_msg      = '';
my $WARN_tra       = ''; 
my $CRIT_tra       = '';
my $status1        = '';
my $status2        = '';
my $perf_data      = '';
my $perf_data_min  = '';
my $perf_data_max  = '';
my $perf_data_uom  = '';
my $perf_data_warn = '';
my $perf_data_crit = '';
my $TYPE_min       = '';
my $TYPE_max       = '';
my $TYPE_uom       = '';

BEGIN {
    $PATH_PL = `dirname $0`;
    $PATH_PL = substr($PATH_PL, 0, -1);
    if (substr($PATH_PL,0,2) eq "./")
        {
        $PATH_PL=substr($PATH_PL,2);
        }
}

use lib "${PATH_PL}";
# sub routines are in sub file
require "check_nut_sub.pl";

$plugin_nagios->add_arg(
spec     => 'type|t=s',
help     => '<type : load, battery, time or status',    # Help of option
required => 1,                  # required
);

$plugin_nagios->add_arg(
spec     => 'lang|l=s',
help     => 'fr or en',    # Help of option
required => 0,                  # optional
);

$plugin_nagios->add_arg(
spec     => 'ini_path|i=s',
help     => 'ini path',    # Help of option
required => 0,                  # optional
);
$plugin_nagios->add_arg(
spec     => 'warning|w=s',
help     => 'value warning state',    # Help of option
required => 0,                  # optional
);

$plugin_nagios->add_arg(
spec     => 'critical|c=s',
help     => 'value critical state',    # Help of option
required => 0,                  # optional
);

$plugin_nagios->getopts();

$TYPE         = $plugin_nagios->opts->get('type');
$WARN         = $plugin_nagios->opts->get('warning');
$CRIT         = $plugin_nagios->opts->get('critical');
$COUNTRY_CODE = $plugin_nagios->opts->get('lang');
$INI_PATH     = $plugin_nagios->opts->get('ini_path');

if (defined($INI_PATH))
    {
    $path_to_ini=$INI_PATH;
    }

# check parameter : type must be load, batterie,temps ou statut
if (($TYPE ne "load") &&
    ($TYPE ne "battery") &&
    ($TYPE ne "time") &&
    ($TYPE ne "status"))
    {
    print "Type must be one of these : load, battery, time or status>";
    exit 3;
    }

# Si temps, il faut definir warning et critique
if ($TYPE eq "load")
{
    if (!(defined($WARN)))
    {
        &display_usage("--warning or -w for warning value.");
        exit 3;
    }
    if (!(defined($CRIT)))
    {
        &display_usage("--critical or -c for critical value.");
        exit 3;
    }
}  

# ---- end parameters retrieve

# check if translation file exists
# if not, try to use default one

if (!(defined($COUNTRY_CODE)))
{
    $COUNTRY_CODE = $country_code_def;
}

# retrieve name of pl script
$PROGNAME=`basename $0`;
chomp($PROGNAME);


# retrieve the path of the script
$PATH=get_ini_path($path_to_ini);

# name of ini file is the same of pl script execpt the extension (.pl)
$NAME_INI=substr($PROGNAME, 0, -3);

# ini_file content the path & name of the ini file
$ini_file = "${PATH}${NAME_INI}_${COUNTRY_CODE}.ini";

if (!(-e $ini_file))
{
    # ini file is not beeing found , try with default name
    $ini_file = "${PATH}${NAME_INI}.ini";
    if (!(-e $ini_file))
    {
        print "fatal error : conf file (${ini_file}) not found";
        exit 3;
    } 
}

#put translations in memory
$hash_ref = Config::INI::Reader->read_file($ini_file);

# a partir de là, on peut utiliser le fichier ini (et donc parametrer les messages)
# At this point, we can use the ini file (so, message can be parametrized)


# read name of exe and find its location
$pgm = $hash_ref->{pgm}->{pgm};
# try to locate (locate instruction does not work on Raspberry PI : we'll uer find instead)
$pgm_tot = `find / -iname $pgm 2>/dev/null`;
chomp($pgm_tot);

# test if the program is installed
if (!(-e $pgm_tot))
{
    $text01a=$hash_ref->{error}->{error01a};                
    $text01b=$hash_ref->{error}->{error01b};                
    print "$text01a '$pgm' $text01b\n";
    exit(3);
}

# if type is battery, we read warning and critical values
# if type is time, only critical value is needed (warning value is 2 times the critical value)

# read ups name
$ups_name = $hash_ref->{pgm}->{ups_name};
# read server IP
$server_IP = $hash_ref->{pgm}->{server_IP};

# concatenation between name and IP (if exists)
$ups_total = '';
if ( $server_IP eq '') {$ups_total = "$ups_name"} else {$ups_total = "$ups_name\@$server_IP"};


$arg = '';
switch ($TYPE) {
    case "load" { $arg = 'ups.load'}
    case "battery" { $arg  = 'battery.charge'; 
                     $WARN = `$pgm_tot $ups_total battery.charge.warning`;
                     $CRIT = `$pgm_tot $ups_total battery.charge.low`;
                   }
    case "time" { $arg  = 'battery.runtime'; 
                  $CRIT = `$pgm_tot $ups_total battery.runtime.low`;
                  # warning value is2 times  critical value
                  $WARN = $CRIT * 2 ;
                }
    case "status" { $arg = 'ups.status'; }
    else { print "type ($TYPE) inconnu"; exit 3; }
}

# call to retrieve current value
$res = `$pgm_tot $ups_total $arg`;

#delete CR at the end
chomp($res);

$res_tra = '';
if ($TYPE eq "time")
{
   # if type is time, we convert seconds into hours, min, seconds
   $temp_tra = $res;
   $nb_hours = int($res / 3600);
   $nb_mins = int(($res - ($nb_hours * 3600))/ 60);
   $res_tra = "$nb_hours h $nb_mins mn";
}
else
{
    $res_tra = $res;
}



# test if value is above warning/critical value

# If type = load, value must be under trigger values --> we toggle the logicial to be as the other type (new value is 100 - read value)
# If type = battery ou times, value must be above trigger value

if ($TYPE eq "load") { $res = 100 - $res; $WARN_tra = 100 - $WARN; $CRIT_tra = 100 - $CRIT} else {$WARN_tra = $WARN; $CRIT_tra = $CRIT}

# the statut as specific processing

if ($TYPE eq "status")
{
    # type is status

    # -- only for test
    # on fait croire qu'on est en critique
    #$res = "$res DISCHRG XX";

    #OL = On Line green / OB = On Battery orange / LB = Low Battery red
    #OL DISCHRG : test in progress --> orange

    # If LB is in the result, critical is send
    if (index($res_tra,"LB") >= 0)
    {
        # critical value is send
        $notif_msg = $hash_ref->{notification_level}->{NLC};
        $result_msg = $hash_ref->{notification_msg}->{NM_STA_C};
        print "$notif_msg : $result_msg";
        exit 2;
    }

    # If OB is in the result, warning is send
    if (index($res_tra,"OB") >= 0)
    {
        # Warning state
        $notif_msg = $hash_ref->{notification_level}->{NLW};
        $result_msg = $hash_ref->{notification_msg}->{NM_STA_W1};
        print "$notif_msg : $result_msg";
        exit 1;
    }

    # If DISCHRG is in the result, warning is send
    if (index($res_tra,"DISCHRG") >= 0)
    {
        # Warning state
        $notif_msg = $hash_ref->{notification_level}->{NLW};
        $result_msg = $hash_ref->{notification_msg}->{NM_STA_W2};
        print "$notif_msg : $result_msg";
        exit 1;
    }

    # Ok state
    $notif_msg = $hash_ref->{notification_level}->{NL0};
    $result_msg = $hash_ref->{notification_msg}->{NM_STA_0};
    print "$notif_msg : $result_msg";
    exit 0;
}
else
{
    # all types but status
    # the value must be under trigger values

    #persormance data --> (sortie | perf data)'label'=value[UOM];[warn];[crit];[min];[max]

    # retrieve if performance data is requiered
    if ($hash_ref->{perf_data}->{$TYPE} eq "y")
    {
        # retrieve min max values
        $TYPE_min = "${TYPE}_min";
        $TYPE_max = "${TYPE}_max";
        $TYPE_uom = "${TYPE}_uom";
        $perf_data_min =$hash_ref->{perf_data}->{$TYPE_min};
        $perf_data_max =$hash_ref->{perf_data}->{$TYPE_max};
        $perf_data_uom=$hash_ref->{perf_data}->{$TYPE_uom};
        if (!(defined($perf_data_uom)))
        {
            $perf_data_uom = '';
        }
        $perf_data_warn=$WARN;
        chomp($perf_data_warn);
        $perf_data_crit=$CRIT;
        chomp($perf_data_crit);
    }



    if ($res < $CRIT_tra)
    {
        $status1=$hash_ref->{status}->{statusc1};
        $status2=$hash_ref->{status}->{statusc2};
        print "$TYPE : $status1 ($res_tra) $status2 ($CRIT).";
        exit 2;
    }

    if ($res < $WARN_tra)
    {
        $status1=$hash_ref->{status}->{statusw1};
        $status2=$hash_ref->{status}->{statusw2};
        print "$TYPE : $status1 ($res_tra) $status2 ($WARN).";
        exit 1;
    }


    $status1=$hash_ref->{status}->{statuso1};
    $status2=$hash_ref->{status}->{statuso2};
    if ($hash_ref->{perf_data}->{$TYPE} eq "y")
    {
        # add data perf
        $perf_data = "|'$TYPE'=$res$perf_data_uom;$perf_data_warn;$perf_data_crit;$perf_data_min;$perf_data_max";
    }
    print "$TYPE : $status1 $res_tra$status2$perf_data";
}

exit 0;


